/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw {


using org.zaval.data.events;
using org.zaval.data;
using org.zaval.port.cs;
using org.zaval.util;


public class LwTextRender:LwRender,TextListener
{
  private FontMetrics fontMetrics;
  private int         textWidth, textHeight;
  private Color       fore;
  private int         startLine, lines;
  private Validationable owner;

  public LwTextRender(string text):this(new Text(text)) {
   
  }

  public LwTextRender(TextModel text):base(text) {
   
  }

  public int getLineHeight() {
    return getFontMetrics().getHeight();
  }

  public void setFont(Font f)
  {
    Font old = getFont();
    if (f != old && (f == null || !f.Equals(old)))
    {
      fontMetrics = LwToolkit.getFontMetrics(f);
      invalidate(0, getTextModel().getSize());
    }
  }

  public Font getFont() {
    return fontMetrics == null?LwToolkit.FONT:fontMetrics.getFont();
  }

  public FontMetrics getFontMetrics() {
    return fontMetrics == null?LwToolkit.FONT_METRICS:fontMetrics;
  }

  public Color getForeground() {
    return fore == null?LwToolkit.FG_COLOR:fore;
  }

  public void setForeground(Color c) {
    if (c != fore && (c == null || !c.Equals(fore)))
      fore = c;
  }

  public virtual int getLineIndent() {
    return 1;
  }

  public override void targetWasChanged(object o, object n)
  {
    if (o != null) ((TextModel)o).removeTextListener(this);
    if (n != null)
    {
      ((TextModel)n).addTextListener(this);
      invalidate(0, getTextModel().getSize());
    }
  }

  public override Dimension calcPreferredSize() {
    return new Dimension(textWidth, textHeight);
  }

  public TextModel getTextModel() {
    return (TextModel)getTarget();
  }

  public string getText() {
    TextModel text = getTextModel();
    return text == null?null:text.getText();
  }

  public override void invalidate() {
    if (owner != null) owner.invalidate();
    base.invalidate();
  }

  public override void recalc ()
  {
    TextModel text = getTextModel();
    if (text != null)
    {
      if (lines > 0)
      {
        for (int i=startLine; i<startLine + lines; i++)
          text.setExtraChar(i, stringWidth(text.getLine(i)));
        lines = 0;
      }

      int lineHeight = getLineHeight();
      int max = 0, size = text.getSize();
      for (int i=0; i<size; i++)
      {
        int len = text.getExtraChar(i);
        if (len > max) max = len;
      }
      textWidth  = max;
      textHeight = lineHeight * size + (size - 1) * getLineIndent();
    }
  }

  public void textRemoved(TextEvent e) {
    if (e.getUpdatedLines() == 0) lines = 0;
    invalidate(e.getFirstUpdatedLine(), e.getUpdatedLines());
  }

  public void textUpdated(TextEvent e) {
    invalidate(e.getFirstUpdatedLine(), e.getUpdatedLines());
  }

  public void textInserted (TextEvent e)
  {
    int ln = e.getUpdatedLines(), first = e.getFirstUpdatedLine();
    if (ln > 1 && lines > 0)
    {
      if (first <= startLine) startLine += (ln - 1);
      else
      if (first < startLine + lines) lines += (ln - 1);
    }
    invalidate(first, ln);
  }

  public void invalidate(int start, int size)
  {
    if ((startLine != start || size != lines) && size > 0)
    {
      if (lines == 0)
      {
        startLine = start;
        lines     = size;
      }
      else
      {
        int e1 = start     + size;
        int e2 = startLine + lines;
        startLine = System.Math.Min(start, startLine);
        lines     = System.Math.Max(e1, e2) - startLine;
      }
    }
    invalidate();
  }


  public virtual void paintLine(Graphics g, int x, int y, int line, Drawable d) {
    g.drawString(getLine(line), x, y + getAscent());
  }

  public override void paint(Graphics g, int x, int y, int w, int h, Drawable d)
  {
     TextModel text   = getTextModel();
     bool b        = d.isEnabled();
     int   lineIndent = getLineIndent();
     int   lineHeight = getLineHeight();
     int   size       = text.getSize();

     Rectangle clip = g.getClipBounds();
     if (clip == null) return;

     w = System.Math.Min(clip.width, w);
     h = System.Math.Min(clip.height, h);

     int startLine = -1;
     if (y < clip.y)
     {
       startLine = (lineIndent + clip.y - y)/(lineHeight + lineIndent);
       h += (clip.y - startLine * lineHeight - startLine * lineIndent);
     }
     else
     if (y > (clip.y + clip.height)) return;
     else startLine = 0;

     if (startLine < size)
     {
       int lines = (h + lineIndent)/(lineHeight + lineIndent) +
                  (((h + lineIndent)%(lineHeight + lineIndent)>lineIndent)?1:0);
       if (startLine + lines > size) lines = size - startLine;
       y += startLine*(lineHeight + lineIndent);

       g.setFont (getFont());
       if (b)
       {
         for (int i=0; i<lines; i++)
         {
           g.setColor(getForeground());
           paintLine(g, x, y, i + startLine, d);
           y += (lineIndent + lineHeight);
         }
       }
       else
       {
         for (int i=0; i<lines; i++)
         {
           g.setColor(Color.gray);
           paintLine(g, x, y, i + startLine, d);
           g.setColor(Color.white);
           paintLine(g, x + 1, y + 1, i + startLine, d);
           y += (lineIndent + lineHeight);
         }
       }
     }
  }

  public int getAscent() {
    return getFontMetrics().getAscent();
  }

  public virtual int lineWidth (int line) {
    validate();
    return getTextModel().getExtraChar(line);
  }

  public virtual int stringWidth (string s) {
    return getFontMetrics().stringWidth(s);
  }

  public virtual int substrWidth (string s, int off, int len) {
    return getFontMetrics().charsWidth(s.ToCharArray(), off, len);
  }

  public virtual string getLine(int r) {
    return getTextModel().getLine(r);
  }

  public override void ownerChanged(Validationable v) {
    owner = v;
  }
}



}
