/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.lw.mask {


using org.zaval.data.events;
using org.zaval.data;
using org.zaval.port.cs;


public class MaskedText:TextModel
{
  private MaskElement[] elements = new MaskElement[0];
  private MaskValidator validator;
  private System.Text.StringBuilder mask = new System.Text.StringBuilder();
  private int extra;
  private TextListenerSupport support;

  public MaskedText(string mask):this(mask, new BasicMaskValidator()) {
   
  }

  public MaskedText(string mask, MaskValidator v) {
    validator = v;
    setMask(mask);
  }

  public string getMask () {
    return mask.ToString();
  }

  public void setMask (string m) {
    if (!m.Equals(getMask())) resetMask(m);
  }

  public string getLine (int i) {
    return getText();
  }

  public MaskValidator getValidator() {
    return validator;
  }

  public void setValidator(MaskValidator v)
  {
    if (v != validator)
    {
      validator = v;
      resetMask(getMask());
    }
  }

  public string getText ()
  {
    System.Text.StringBuilder buf = new System.Text.StringBuilder();
    for (int i=0; i<elements.Length; i++) buf.Append(elements[i].value);
    return buf.ToString();
  }

  public void setText (string t) {
    setText (0, t);
  }

  public int getSize () {
    return 1;
  }

  public int getTextLength () {
    return mask.Length;
  }

  public void write(char ch, int offset) {
    write((ch).ToString(), offset);
  }

  public void write(string s, int offset) {
    setText(offset, s);
  }

  public void remove (int offset, int count)
  {
    int[] res = findElementByOffset(offset);
    int k = res[1], c = count;
    for (int j = res[0]; j < elements.Length && count > 0; j++)
    {
      if (elements[j].getType() != -1)
      {
        for (; k < elements[j].size && count > 0; k++, count--)
          if (validator.isHandledTag(elements[j].getTagAt(k)))
            elements[j].value[k] = validator.getBlankChar(elements[j].getTagAt(k));
      }
      else count -= (elements[j].size - k);
      k = 0;
    }
    if (c > 0) perform (TextEvent.UPDATED, offset, c);
  }

  public void addTextListener(TextListener l) {
    if (support == null) support = new TextListenerSupport();
    support.addListener(l);
  }

  public void removeTextListener(TextListener l) {
    if (support != null) support.removeListener(l);
  }

  public int getExtraChar (int i) {
    return extra;
  }

  public void setExtraChar (int i, int ch) {
    extra = ch;
  }

  public int findHandledTag (int offset, int d)
  {
    while (offset + d > 0 && offset + d < getTextLength() && !isHandledOffset(offset))
      offset+=d;
    return offset;
  }


  public bool isHandledOffset (int offset) {
    return validator.isHandledTag(mask[offset]);
  }

  void setText (int offset, string value)
  {
    int[]  res = findElementByOffset(offset);
    char[] buf = value.ToCharArray();
    int i = 0, j = res[0], k = res[1];
    while (i < buf.Length && j < elements.Length)
    {
      if (elements[j].getType() != -1)
      {
        int    len = System.Math.Min(elements[j].getSize() - k, value.Length - i);
        char[] tmp = elements[j].getValue().ToCharArray();
        System.Array.Copy (buf, i, tmp, k, len);
        string newValue = validator.completeValue(elements[j], new string(tmp));
        if (validator.isValidValue(elements[j], newValue)) elements[j].value = newValue.ToCharArray();
        else break;
        i += len;
      }
      else
        for (;k < elements[j].size && i < buf.Length && buf[i] == elements[j].getTagAt(k);k++);
      j++;
      k = 0;
    }


    if (i > 0) perform (TextEvent.UPDATED, offset, i);
  }

  void resetMask (string m)
  {
    if (mask.Length > 0)
    {
      System.Text.StringBuilder prev = mask;
      mask     = new System.Text.StringBuilder();
      elements = new MaskElement[0];
      perform (TextEvent.REMOVED, 0, prev.Length);
    }

    mask = new System.Text.StringBuilder(m);
    Vector res = new Vector();
    int index = 0, length = mask.Length;
    while (index < length)
    {
      int prevIndex = index;
      int elementType = validator.getTypeByTag(mask[index]);
      while (index + 1 < length && elementType == validator.getTypeByTag(mask[index + 1]))
        index++;

      MaskElement el = new MaskElement();
      el.type  = elementType;
      el.tags  = m.Substring(prevIndex, index + 1 - prevIndex).ToCharArray();
      el.size  = index - prevIndex + 1;
      el.value = new char[el.size];
      for (int i=0; i<el.size; i++)
        el.value[i] = (validator.isHandledTag(el.tags[i]))?validator.getBlankChar(el.tags[i]):el.tags[i];
      index++;
      res.addElement(el);
    }

    elements = new MaskElement[res.size()];
    for (int i=0; i<elements.Length; i++) elements[i] = (MaskElement)res.elementAt(i);
    perform (TextEvent.INSERTED, 0, mask.Length);
  }

  void perform (int id, int offset, int size)
  {
    if (support != null)
    {
      TextEvent e = new TextEvent(this, id, offset, size);
      e.setUpdatedLines(0, 1);
      support.perform(e);
    }
  }

  private int[] findElementByOffset(int offset)
  {
    int off = 0, j = 0;
    for (j=0;j<elements.Length; j++)
    {
      if (offset >= off && offset < elements[j].size + off) break;
      off += elements[j].size;
    }
    return new int[] {j, offset-off};
  }
}



}
