/**
 *     Caption: Zaval Light-Weight Visual Components Library
 *     $Revision: 2.79 $
 *     $Date: 2003/08/22 11:24:16 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    3.50
 *
 * Zaval Light-Weight Visual Components Library (LwVCL) is a pure Java
 * alternative to humble AWT-based and SWING-based GUI interfaces for
 * wide ranges of platforms, including J2SE, PersonalJava and J2ME.
 *
 * Designed as light-weight but, alternatively to Swing, built separately
 * from AWT (not on top of the java.awt library like Swing), the LwVCL is
 * the good alternative to highly performant, memory-efficient, flexible
 * GUI solution for embedded, stand-alone and applet applications.
 *
 * For more info on this product read Zaval Light-Weight Visual Components Library Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/lwvcl/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0003/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2003  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
namespace org.zaval.misc {


using org.zaval.misc.events;
using org.zaval.port.cs;


public class PosController:org.zaval.util.ValidationObject
{
  public const  int DOWN = 1;

  public const  int UP = 2;

  public const  int BEG = 3;

  public const  int END = 4;

  private int currentLine, currentCol, offset;
  private PosInfo pi;
  private Vector support;
  public int prevOffset, prevCol, prevLine;

  public PosController() {}

  public PosController(PosInfo pi) {
    setPosInfo(pi);
  }

  public void setPosInfo(PosInfo p)
  {
    if (p == null) throw new org.zaval.port.cs.IllegalArgumentException();
    if (p != pi)
    {
      pi = p;
      clearPos();
    }
  }

  public virtual void clearPos()
  {
    if (offset >= 0)
    {
      validate();
      prevOffset = offset;
      prevLine   = currentLine;
      prevCol    = currentCol;
      offset = -1;
      currentLine = -1;
      currentCol = -1;
      perform(new PosEvent(this,  prevOffset, prevLine, prevCol));
    }
  }

  public virtual void setOffset (int o)
  {
    if (o < 0) o = 0;
    else
    {
      int max = getMaxOffset();
      if (o >= max) o = max;
    }

    if (o != offset)
    {
      validate();
      prevOffset = offset;
      prevLine = currentLine;
      prevCol  = currentCol;
      offset = o;
      invalidate();
      perform(new PosEvent(this, prevOffset, prevLine, prevCol));
    }
  }

  public virtual void seek(int off) {
    setOffset(offset + off);
  }

  public void setRowCol (int r, int c)
  {
    if (r != currentLine || c != currentCol)
    {
      prevOffset  = offset;
      prevLine    = currentLine;
      prevCol     = currentCol;
      currentLine = r;
      currentCol  = c;
      offset = getOffsetByPoint(r, c, this);
      perform(new PosEvent(this,  prevOffset, prevLine, prevCol));
    }
  }

  public void seekLineTo(int t) {
    seekLineTo (t, 1);
  }

  public void seekLineTo(int t, int num)
  {
     if (getOffset() < 0)
     {
       setOffset(0);
       return;
     }

     validate();
     prevOffset = offset;
     prevLine   = currentLine;
     prevCol    = currentCol;
     switch (t)
     {
       case BEG :
       {
         if (currentCol > 0)
         {
           offset -= currentCol;
           currentCol = 0;
           perform(new PosEvent(this, prevOffset, prevLine, prevCol));
         }
       } break;
       case END :
       {
         int maxCol = pi.getLineSize(currentLine);
         if (currentCol < (maxCol - 1))
         {
           offset += (maxCol - currentCol - 1);
           currentCol = maxCol - 1;
           perform(new PosEvent(this, prevOffset, prevLine, prevCol));
         }
       } break;
       case UP :
       {
         if (currentLine > 0)
         {
           offset -= (currentCol + 1);
           currentLine--;
           for (int i=0; currentLine > 0 && i < (num - 1); i++, currentLine--)
             offset -= pi.getLineSize(currentLine);

           int maxCol = pi.getLineSize(currentLine);
           if (currentCol < maxCol) offset -= (maxCol - currentCol - 1);
           else                     currentCol = maxCol - 1;
           perform(new PosEvent(this, prevOffset, prevLine, prevCol));
         }
       } break;
       case DOWN :
       {
         if (currentLine < (pi.getLines() - 1))
         {
           offset += (pi.getLineSize(currentLine) - currentCol);
           currentLine++;
           int size = pi.getLines() - 1;
           for (int i=0; currentLine < size && i < (num-1); i++, currentLine++)
            offset += pi.getLineSize(currentLine);

           int maxCol = pi.getLineSize(currentLine);
           if (currentCol < maxCol) offset += currentCol;
           else
           {
             offset += (maxCol - 1);
             currentCol = maxCol - 1;
           }
           perform(new PosEvent(this, prevOffset, prevLine, prevCol));
         }
       } break;
       default : throw new org.zaval.port.cs.IllegalArgumentException();
     }
  }

  public int getMaxOffset()
  {
    int max = pi.getMaxOffset();
    if (max < 0)
    {
      validate();
      return calcMaxOffset(pi);
    }
    else  return max;
  }

  public int getCurrentCol() {
    validate();
    return currentCol;
  }

  public int getCurrentLine() {
    validate();
    return currentLine;
  }

  public int getOffset() {
    return offset;
  }

  public override void recalc()
  {
    if (getOffset() >= 0)
    {
      Point p = getPointByOffset(getOffset(), this);
      currentLine = p.x;
      currentCol  = p.y;
    }
  }

  public void addPosListener (PosListener l) {
    if (support == null) support = new Vector(1);
    if (!support.contains(l)) support.addElement(l);
  }

  public void removePosListener (PosListener l) {
    if (support != null) support.removeElement(l);
  }

  public PosInfo getPosInfo() {
    return pi;
  }

  public virtual void perform (PosEvent e)
  {
    if (support != null)
     for (int i=0;i<support.size();i++)
      ((PosListener)support.elementAt(i)).posChanged(e);
  }

  public static Point getPointByOffset(int offset, PosController pc)
  {
     PosInfo pi = pc.getPosInfo();
     if (pi == null) return null;

     if (offset > pc.prevOffset)
     {
       bool b = pc.prevOffset < 0;
       int startOffset = b?0:pc.prevOffset - pc.prevCol;
       int startLine   = b?0:pc.prevLine;
       for (int i=startLine; i < pi.getLines(); i++)
       {
         int l = pi.getLineSize(i);
         if (offset >= startOffset && offset <  startOffset + l) return new Point(i, offset - startOffset);
         startOffset += l;
       }
     }
     else
     if (offset < pc.prevOffset)
     {
       bool b = (pc.prevLine >= pi.getLines() || pc.prevOffset > pi.getMaxOffset());
       int startLine = b?pi.getLines() - 1:pc.prevLine;
       int endOffset = b?pi.getMaxOffset():pc.prevOffset + (pi.getLineSize(startLine) - pc.prevCol - 1);
       for (int i=startLine; i >= 0; i--)
       {
         int l = pi.getLineSize(i);
         if (offset <= endOffset && offset >=  endOffset - l + 1) return new Point(i, l - (endOffset - offset + 1));
         endOffset -= l;
       }
     }
     return new Point (pc.prevLine, pc.prevCol);
  }

  public static int getOffsetByPoint(int row, int col, PosController pc)
  {
    PosInfo pi = pc.getPosInfo();
    int startOffset = pc.prevOffset < 0?0:pc.prevOffset - pc.prevCol;
    int startLine   = pc.prevOffset < 0?0:pc.prevLine;

    if (startLine <= row)
    {
      for (int i = startLine; i<row; i++)
        startOffset += pi.getLineSize(i);
    }
    else
    {
      for (int i = startLine-1; i>=row; i--)
        startOffset -= pi.getLineSize(i);
    }
    return startOffset + col;
  }

  public static int calcMaxOffset(PosInfo pi)
  {
    int max = 0;
    for (int i = 0; i < pi.getLines(); i++) max += pi.getLineSize(i);
    return max - 1;
  }
}



}
